<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package GA4 Hyva Compatibility
 */

namespace Amasty\GA4Hyva\Model\Event\Catalog\Logger\ItemListsProcessors;

use Amasty\GA4\Model\Event\Catalog\Logger\ItemListsProcessors\Crosssell as CrosssellOriginal;
use Amasty\GA4\Model\Event\Catalog\Logger\ItemListsProcessors\Common;
use Amasty\GA4\Model\Utils\ProductDetailsFormatter;
use Hyva\Theme\Service\CurrentTheme;
use Hyva\Theme\ViewModel\Cart\Items as CartItems;
use Hyva\Theme\ViewModel\ProductList;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product\Visibility as ProductVisibility;
use Magento\Framework\View\LayoutInterface;
use Magento\Store\Model\StoreManagerInterface;

class Crosssell extends Common
{
    public const PROCESSOR_TYPE = 'crosssell';
    public const ITEM_LIST_ID = 'crosssell_products';

    /**
     * @var ProductDetailsFormatter
     */
    private ProductDetailsFormatter $productDetailsFormatter;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    /**
     * @var LayoutInterface
     */
    private LayoutInterface $layout;

    /**
     * @var ProductList
     */
    private ProductList $productList;

    /**
     * @var CartItems
     */
    private CartItems $cartItems;

    /**
     * @var CurrentTheme
     */
    private CurrentTheme $currentTheme;

    /**
     * @var CrosssellOriginal
     */
    private CrosssellOriginal $crosssellOriginal;

    /**
     * @var int
     */
    private int $productSize;

    /**
     * @param ProductDetailsFormatter $productDetailsFormatter
     * @param StoreManagerInterface $storeManager
     * @param LayoutInterface $layout
     * @param ProductList $productList
     * @param CartItems $cartItems
     * @param CurrentTheme $currentTheme
     * @param CrosssellOriginal $crosssellOriginal
     * @param int $productSize
     */
    public function __construct(
        ProductDetailsFormatter $productDetailsFormatter,
        StoreManagerInterface $storeManager,
        LayoutInterface $layout,
        ProductList $productList,
        CartItems $cartItems,
        CurrentTheme $currentTheme,
        CrosssellOriginal $crosssellOriginal,
        int $productSize = 8
    ) {
        parent::__construct($layout);

        $this->productDetailsFormatter = $productDetailsFormatter;
        $this->storeManager = $storeManager;
        $this->layout = $layout;
        $this->productList = $productList;
        $this->cartItems = $cartItems;
        $this->currentTheme = $currentTheme;
        $this->crosssellOriginal = $crosssellOriginal;
        $this->productSize = $productSize;
    }

    /**
     * Check if event data is allowed for current page
     *
     * @return bool
     */
    public function isAvailable(): bool
    {
        if (!$this->currentTheme->isHyva()) {
            return $this->crosssellOriginal->isAvailable();
        }

        return $this->layout->hasElement(static::PROCESSOR_TYPE);
    }

    /**
     * Prepare event data result for output
     *
     * @return array
     */
    public function processViewList(): array
    {
        if (!$this->currentTheme->isHyva()) {
            return $this->crosssellOriginal->processViewList();
        }

        return parent::processViewList();
    }

    /**
     * Prepare event data and format the result
     *
     * @return array
     */
    protected function prepareProductsEventData(): array
    {
        $products = array_values($this->getProducts());

        if (empty($products)) {
            return [];
        }

        $result = [];
        foreach ($products as $index => $product) {
            $result[$product->getId()] = $this->productDetailsFormatter->formatFromProduct(
                $product,
                self::ITEM_LIST_ID,
                __('Cross-Sell Products')->render(),
                $index + 1
            );
        }

        return $result;
    }

    /**
     * Get products for event data
     *
     * @return ProductInterface[]
     */
    private function getProducts(): array
    {
        $cartItems = $this->cartItems->getCartItems();

        $currentWebsite = $this->storeManager->getWebsite();

        $this->productList->setPageSize($this->productSize);
        $this->productList->addFilter('website_id', $currentWebsite->getId());
        $this->productList->addFilter('visibility', [
            ProductVisibility::VISIBILITY_IN_CATALOG,
            ProductVisibility::VISIBILITY_IN_SEARCH,
            ProductVisibility::VISIBILITY_BOTH,
        ], 'in');

        return $this->productList->getCrosssellItems(...$cartItems);
    }
}
