<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package GA4 Hyva Compatibility
 */

namespace Amasty\GA4Hyva\Model\Event\Catalog\Logger\ItemListsProcessors;

use Amasty\GA4\Model\Event\Catalog\Logger\ItemListsProcessors\Related as RelatedOriginal;
use Amasty\GA4\Model\Event\Catalog\Logger\ItemListsProcessors\Common;
use Amasty\GA4\Model\Utils\ProductDetailsFormatter;
use Hyva\Theme\Service\CurrentTheme;
use Hyva\Theme\ViewModel\CurrentProduct;
use Hyva\Theme\ViewModel\ProductList;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product\Visibility as ProductVisibility;
use Magento\Framework\Escaper;
use Magento\Framework\View\LayoutInterface;
use Magento\Store\Model\StoreManagerInterface;

class Related extends Common
{
    public const PROCESSOR_TYPE = 'related';
    public const ITEM_LIST_ID = 'related_products';

    /**
     * @var ProductDetailsFormatter
     */
    private ProductDetailsFormatter $productDetailsFormatter;

    /**
     * @var StoreManagerInterface
     */
    private StoreManagerInterface $storeManager;

    /**
     * @var LayoutInterface
     */
    private LayoutInterface $layout;

    /**
     * @var Escaper
     */
    private Escaper $escaper;

    /**
     * @var ProductList
     */
    private ProductList $productList;

    /**
     * @var CurrentProduct
     */
    private CurrentProduct $currentProduct;

    /**
     * @var CurrentTheme
     */
    private CurrentTheme $currentTheme;

    /**
     * @var RelatedOriginal
     */
    private RelatedOriginal $relatedOriginal;

    /**
     * @var int
     */
    private int $productSize;

    /**
     * @param ProductDetailsFormatter $productDetailsFormatter
     * @param StoreManagerInterface $storeManager
     * @param LayoutInterface $layout
     * @param Escaper $escaper
     * @param ProductList $productList
     * @param CurrentProduct $currentProduct
     * @param CurrentTheme $currentTheme
     * @param RelatedOriginal $relatedOriginal
     * @param int $productSize
     */
    public function __construct(
        ProductDetailsFormatter $productDetailsFormatter,
        StoreManagerInterface $storeManager,
        LayoutInterface $layout,
        Escaper $escaper,
        ProductList $productList,
        CurrentProduct $currentProduct,
        CurrentTheme $currentTheme,
        RelatedOriginal $relatedOriginal,
        int $productSize = 8
    ) {
        parent::__construct($layout);

        $this->productDetailsFormatter = $productDetailsFormatter;
        $this->storeManager = $storeManager;
        $this->layout = $layout;
        $this->escaper = $escaper;
        $this->productList = $productList;
        $this->currentProduct = $currentProduct;
        $this->currentTheme = $currentTheme;
        $this->relatedOriginal = $relatedOriginal;
        $this->productSize = $productSize;
    }

    /**
     * Check if event data is allowed for current page
     *
     * @return bool
     */
    public function isAvailable(): bool
    {
        if (!$this->currentTheme->isHyva()) {
            return $this->relatedOriginal->isAvailable();
        }

        return $this->layout->hasElement(static::PROCESSOR_TYPE);
    }

    /**
     * Prepare event data result for output
     *
     * @return array
     */
    public function processViewList(): array
    {
        if (!$this->currentTheme->isHyva()) {
            return $this->relatedOriginal->processViewList();
        }

        return parent::processViewList();
    }

    /**
     * Prepare event data and format the result
     *
     * @return array
     */
    protected function prepareProductsEventData(): array
    {
        $products = array_values($this->getProducts());

        if (empty($products)) {
            return [];
        }

        $listName = __('Related Products');
        if ($product = $this->currentProduct->get()) {
            $listName = __('Related Products from %1', $this->escaper->escapeHtml($product->getName()));
        }

        $result = [];
        foreach ($products as $index => $product) {
            $result[$product->getId()] = $this->productDetailsFormatter->formatFromProduct(
                $product,
                self::ITEM_LIST_ID,
                $listName->render(),
                $index + 1
            );
        }

        return $result;
    }

    /**
     * Get products for event data
     *
     * @return ProductInterface[]
     */
    public function getProducts(): array
    {
        $product = $this->currentProduct->get();

        $currentWebsite = $this->storeManager->getWebsite();

        $this->productList->setPageSize($this->productSize);
        $this->productList->addFilter('website_id', $currentWebsite->getId());
        $this->productList->addFilter('visibility', [
            ProductVisibility::VISIBILITY_IN_CATALOG,
            ProductVisibility::VISIBILITY_IN_SEARCH,
            ProductVisibility::VISIBILITY_BOTH,
        ], 'in');

        return $this->productList->getRelatedItems($product);
    }
}
