<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package GA4 Hyva Compatibility
 */
declare(strict_types=1);

namespace Amasty\GA4Hyva\Observer;

use Amasty\GA4\Model\Event\Cart\RemoveFromCart;
use Amasty\GA4\Model\EventsProcessor;
use Amasty\GA4Api\Api\Event\EventConfigInterface;
use Amasty\GA4Api\Model\DataLayer\DataLayerStorageInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\App\RequestInterface;
use Amasty\GA4\Model\Frontend\GA4Session;

/**
 * Abstract class
 * to create virtualTypes for events
 */
class ProcessEventToSessionObserver implements ObserverInterface
{
    /** @var DataLayerStorageInterface */
    private DataLayerStorageInterface $dataLayerStorage;

    /** @var EventsProcessor */
    private EventsProcessor $eventsProcessor;

    /** @var RequestInterface */
    private RequestInterface $request;

    /** @var GA4Session */
    private GA4Session $ga4Session;

    /** @var EventConfigInterface */
    private EventConfigInterface $event;

    /** @var array */
    private array $skipEventAction;

    /**
     * @param DataLayerStorageInterface $dataLayerStorage
     * @param EventsProcessor $eventsProcessor
     * @param RequestInterface $request
     * @param GA4Session $ga4Session
     * @param EventConfigInterface $event
     * @param array $skipEventAction
     */
    public function __construct(
        DataLayerStorageInterface $dataLayerStorage,
        EventsProcessor $eventsProcessor,
        RequestInterface $request,
        GA4Session $ga4Session,
        EventConfigInterface $event,
        array $skipEventAction = [RemoveFromCart::NAME => ['wishlist_index_fromcart']]
    ) {
        $this->dataLayerStorage = $dataLayerStorage;
        $this->eventsProcessor = $eventsProcessor;
        $this->request = $request;
        $this->ga4Session = $ga4Session;
        $this->event = $event;
        $this->skipEventAction = $skipEventAction;
    }

    /**
     * Event data handler
     *
     * @param Observer $observer
     */
    public function execute(
        Observer $observer
    ): void {
        $eventName = $this->event->getName();

        if (!$this->isValidActionEvent($eventName)) {
            return;
        }

        $params = $this->request->getParams();
        if (empty($params['item_id']) && !empty($params['id'])) {
            $params['item_id'] = $params['id'];
        }

        if ($wishlist = $observer->getData('wishlist')) {
            $params['wishlist_id'] = $wishlist->getId();
        }

        $this->eventsProcessor->processEvent($eventName, $params);

        $collectedData = $this->dataLayerStorage
            ->getDataLayerOptions($eventName);

        $this->ga4Session->addEvent($eventName, $collectedData);
    }

    /**
     * Check if need to add event depends on action
     *
     * @param string $eventName
     * @return bool
     */
    private function isValidActionEvent(string $eventName): bool
    {
        $action = $this->request->getFullActionName();

        return !isset($this->skipEventAction[$eventName]) ||
            !in_array($action, $this->skipEventAction[$eventName], true);
    }
}
